//-----------------------------------------------------
//-----------------------------------------------------
// Pic Pack library
// 
// plt-1004 v4 firmware.c
//
// Demonstration routines showing the draw library
// used with the Embedded Adventures LED matrix display boards
//
// (c) Embedded Adventures 2012
// iharris [at] embeddedadventures.com
//
//
//-----------------------------------------------------

/*

Commands available

recording start/stop
displays x
clear
pixel     colour   x        y
line      colour   x1       y1       x2       y2
circle    colour   x_centre y_centre radius
circle2   colour   x_centre y_centre radius
line      colour   x1       y1       x2     y2
rectangle colour   x        y        width  height
font      font_num
text      colour   x        y        "text"
scroll    colour   x        y        width  "text"
paint

*/

// Pic Pack includes
#include "config.h"
#include "pic_utils.h"
#include "pic_serial.h"
#include "draw\draw.h"
// System includes
#include "string.h"
#include "stdlib.h"
#include "ctype.h"
#include "draw\recording.h"

#if DRAW_DRIVER == LDP6416
	#include "ea_ldp6416.h"
	#define DRAW_DISPLAYS ea_ldp6416_displays
	#define DRAW_ORIENTATION ea_ldp6416_display_orientation
#endif
#if DRAW_DRIVER == LDP6408
	#include "ea_ldp6408.h"
	#define DRAW_DISPLAYS ea_ldp6408_displays
	#define DRAW_ORIENTATION ea_ldp6408_display_orientation
#endif
#if DRAW_DRIVER == LDP3208S
	#include "ea_ldp3208s.h"
	#define DRAW_DISPLAYS ea_ldp3208s_displays
	#define DRAW_ORIENTATION ea_ldp3208s_display_orientation
#endif
#if DRAW_DRIVER == LDP8008
	#include "ea_ldp8008.h"
	#define DRAW_DISPLAYS ea_ldp8008_displays
	#define DRAW_ORIENTATION ea_ldp8008_display_orientation
#endif
#if DRAW_DRIVER == LDP6432
	#include "ea_ldp6432.h"
	#define DRAW_DISPLAYS ea_ldp6432_displays
	#define DRAW_ORIENTATION ea_ldp6432_display_orientation

#endif
#if DRAW_DRIVER == LDP3208S
	#include "ea_ldp3208s.h"
	#define DRAW_DISPLAYS ea_ldp3208s_displays
	#define DRAW_ORIENTATION ea_ldp3208s_display_orientation

#endif
	
#include "draw\draw_screen_buffer.h"
// Support for Timer 0
#include "pic_timer.h"
// Support for Timer 1
#include "pic_timer1.h"
#include "pic_term.h"
#include "draw\draw_tests.h"
#include "pic_tick.h"

#include "draw\draw_fonts.h"

#ifndef BIGNUM_NOT_SUPPORTED
	#include "draw\draw_bignum.h"
#endif

draw_x_type x_offset = 0;
draw_y_type y_offset = 0;

bit recording = 0;
uns8 scroll_speed = 150;

// Interrupt routine - - - - - - - - - -
void interrupt() {
	
	serial_handle_rx_isr();

	// very important for timer 1 handler to be first: It refreshes
	// the display 
	timer_handle_1_isr();	// Handle timer 0 interrupt
	timer_handle_0_isr();	
	serial_handle_tx_isr();
	serial_handle_rx_isr();

	
}

void timer_1_callback() {
	// refresh display from draw buffer

	drv_refresh();	

}

// Timer 0 callback handled by pic_tick

uns8 my_test;

uns8  scroll_string[170];
uns16 scroll_length;
uns16 scroll_x;
uns16 scroll_text_length;
uns16 scroll_start_pixel;
uns8  scroll_mode = 3;
uns8  scroll_colour;
uns8  scroll_font_height;
draw_x_type scroll_width;
draw_y_type scroll_y_pos;
draw_x_type scroll_x_pos;
uns8 font_id = 1;
uns8 displays = 1;

void scroll_setup() {
	scroll_text_length = draw_fonts_length_str(font_id, &scroll_string);
	//serial_print_var("len=", scroll_text_length);
	scroll_x = 0;
	scroll_length = scroll_width + scroll_text_length;
	scroll_mode = 1;	// 1 = we scroll onto the blank screen
	scroll_start_pixel = 0;
	scroll_font_height = draw_fonts_height(font_id);
	//serial_print_var("h=", scroll_font_height);
}


void scroll_display() {
	
	if (scroll_mode == 3) return;
		
	draw_rect(scroll_x_pos, scroll_y_pos, scroll_width, scroll_font_height, 0 );
	//serial_print_str("Done\n");
	//	serial_print_var("sxp=", scroll_x_pos);
	//serial_print_var("sxy=", scroll_y_pos);
	//serial_print_var("sw=", scroll_width);
	//serial_print_var("sfh=", scroll_font_height);
	//serial_print_var("scrollx=", scroll_x);
	if (scroll_mode == 1) {
		draw_fonts_print_str(font_id, /* x */ scroll_x_pos + scroll_width - scroll_x - 1, /* y */ scroll_y_pos, /* width */ scroll_x + 1, /* start_pixel */ 0, /* colour */  scroll_colour, &scroll_string);
		scroll_x++;
		if (scroll_x == scroll_width) {
			scroll_mode = 2;
			//serial_print_str("Now mode 2\n");
		}
	} else { // mode 2
		//serial_print_debug("scroll_x_pos", scroll_x_pos);
		//serial_print_debug("scroll_y_pos", scroll_y_pos);
		//serial_print_debug("scroll_width", scroll_width);
		//serial_print_debug("start_pixel", scroll_x - scroll_width + 1);
		draw_fonts_print_str(font_id, /* x */ scroll_x_pos, /* y */ scroll_y_pos, scroll_width, /* start_pixel */ scroll_x - scroll_width + 1, /* colour */  scroll_colour, &scroll_string);
		scroll_x++;
		if (scroll_x == scroll_length) {
			scroll_mode = 3; 	// done
			serial_print_str("<DONE>\n");
			// add looping
			//scroll_x = 0;
			//scroll_mode = 1;	// 1 = we scroll onto the blank screen
			//scroll_start_pixel = 0;
		}
	}
	draw_paint();
}

void set_displays(uns8 display_num) {
			
			#if DRAW_ORIENTATION == HORIZONTAL
				x_offset = (DRAW_DISPLAYS - display_num) * (DRAW_PIXELS_WIDE / DRAW_DISPLAYS);
				y_offset = 0;
			#else 
				y_offset = (DRAW_DISPLAYS - display_num) * (DRAW_PIXELS_HIGH / DRAW_DISPLAYS);
				x_offset = 0;
			#endif
			serial_print_str("Setting x offset to ");
			serial_print_int(x_offset); 
			serial_print_str(" and y offset to ");
			serial_print_int(y_offset);
			serial_print_str(" to support ");
			serial_print_int(display_num);
			serial_print_str(" displays.\n");
			displays = display_num;
}
void title_screen() {
	draw_x_type width;
	draw_y_type height;
	uns8 col1, col2, col3;
	
	#if DRAW_BITS_PER_PIXEL == 2
		col1 = 3;
		col2 = 1;
		col3 = 2;
	#else
		col1 = 1;
		col2 = 1;
		col3 = 1;
	#endif

	#if DRAW_ORIENTATION == HORIZONTAL
		width = DRAW_PIXELS_WIDE / DRAW_DISPLAYS * displays;
		height = DRAW_PIXELS_HIGH;
	#else 
		width = DRAW_PIXELS_WIDE;
		height = DRAW_PIXELS_HIGH / DRAW_DISPLAYS * displays;
	#endif	
	/*serial_print_str("width = ");
	serial_print_int(width);
	serial_print_str(" height = ");
	serial_print_int(height);
	*/
	draw_clear_screen();
	
	// border
	draw_line(0 + x_offset, 0 + y_offset, width -1 + x_offset, 0 + y_offset, col1);
	draw_line(width - 1 + x_offset, 0 + y_offset, width - 1 + x_offset, height - 1 + y_offset, col1);
	draw_line(width - 1 + x_offset, height - 1 + y_offset, 0 + x_offset, height - 1 + y_offset, col1);
	draw_line(0 + x_offset, height - 1 + y_offset, 0 + x_offset, 0 + y_offset, col1);

	// version
	if (width > 32) {
		draw_fonts_print_str(/*font_id*/ 2, /*x*/ 2 + x_offset, /*y*/ 6 + y_offset, /*width*/ 64, /*start_pixel*/ 0, /*colour*/ col2, "VER:");
		draw_fonts_print_str(/*font_id*/ 2, /*x*/ 24 + x_offset, /*y*/ 6 + y_offset, /*width*/ 64, /*start_pixel*/ 0, /*colour*/ col3, BASE_VERSION);
	} else {
		draw_fonts_print_str(/*font_id*/ 2, /*x*/ 2 + x_offset, /*y*/ 6 + y_offset, /*width*/ 64, /*start_pixel*/ 0, /*colour*/ col3, BASE_VERSION);
	}
	if (height > 8) {
		draw_fonts_print_str(/*font_id*/ 1, /*x*/ 2 + x_offset, /*y*/ 13 + y_offset,  /*width*/ 64, /*start_pixel*/ 0, /*colour*/ col2, "PLT-1001V4");
	}
	if (height > 16) {
		draw_fonts_print_str(/*font_id*/ 2, /*x*/ 2 + x_offset, /*y*/ 22 + y_offset, /*width*/ 64, /*start_pixel*/ 0, /*colour*/ col3, "EMBEDDED");
		draw_fonts_print_str(/*font_id*/ 2, /*x*/ 2 + x_offset, /*y*/ 28 + y_offset, /*width*/ 64, /*start_pixel*/ 0, /*colour*/ col3, "ADVENTURES");
	}
	draw_paint();
}


#define MAX_PARAMS 5

void term_entry_callback(uns8 *term_buffer) {


uns8 first_char;
uns8 *token;

uns16 param_list[MAX_PARAMS];
uns8 param_count;
uns8 token_length;
uns8 *string_param = 0;
uns8 *command;
uns8 *last;
uns8 *second_command;
bit bad_command = 1;
//uns8 record_copy[80];
//	if (recording) {
//		strcpy(record_copy, term_buffer);
//	}	
	param_count = 0;
	serial_print_nl();
	string_param = strchr(term_buffer, '\"');
	if (string_param) {
		*string_param = '\0';
		string_param++;
		last = string_param;
		while (*last) { last++;}
		last--;
		*last = '\0';
	}	
	token = strtok (term_buffer," ,");
	if (token) {
		command = token;
		token = strtok (NULL," ,");
		second_command = token;
		while ((param_count < MAX_PARAMS) && token) {
				param_list[param_count++] = atoui_dec(token);
				token = strtok (NULL," ,");
		}
	}

	
	if (!strcmp(command, "clear")) {
		draw_clear_screen();
 		bad_command = 0;
 	} else if (!strcmp(command, "displays") && (param_count == 1)) {
		if (param_list[0] <= DRAW_DISPLAYS) {
			set_displays(param_list[0]);
			bad_command = 0;
		}	
 	} else if (!strcmp(command, "scrollspeed") && (param_count == 1)) {
			scroll_speed = param_list[0];
			bad_command = 0;
	} else if (!strcmp(command, "record") && (param_count == 1)) {
		serial_print_str("recording?");
		if (!strcmp(second_command, "start")) {
			recording = 1;
			//recording_start();
			bad_command = 0;
		} else if (!strcmp(second_command, "stop")) {
			recording = 0;
			bad_command = 0;
			//recording_stop();
		}	
	} else if (!strcmp(command, "pixel") && (param_count == 3)) {
		draw_set_pixel(param_list[1] + x_offset, param_list[2] + y_offset, param_list[0]);
		bad_command = 0;
	} else if (!strcmp(command, "line") && (param_count == 5)) {
		draw_line(param_list[1] + x_offset, param_list[2] + y_offset, param_list[3] + x_offset, param_list[4] + y_offset, param_list[0]);
		bad_command = 0;
	} else if (!strcmp(command, "circle") && (param_count == 4)) {
		draw_circle(param_list[1] + x_offset, param_list[2] + y_offset, param_list[3], param_list[0]);
		bad_command = 0;
	} else if (!strcmp(command, "circle2") && (param_count == 4)) {
		draw_circle2(param_list[1] + x_offset, param_list[2] + y_offset, param_list[3], param_list[0]);
		bad_command = 0;
	} else if (!strcmp(command, "circlef") && (param_count == 4)) {
		draw_filled_circle(param_list[1] + x_offset, param_list[2] + y_offset, param_list[3], param_list[0]);
		bad_command = 0;
	} else if (!strcmp(command, "rect") && (param_count == 5)) {
		draw_rect(param_list[1] + x_offset, param_list[2] + y_offset, param_list[3], param_list[4], param_list[0]);
		bad_command = 0;
	} else if (!strcmp(command, "font") && (param_count == 1)) {
		if ((param_list[0] > 0) && (param_list[0] < 9)) {
			font_id = param_list[0];
			bad_command = 0;
		}
	} else if (!strcmp(command, "title")) {
		title_screen();
		bad_command = 0;
	} else if (!strcmp(command, "text") && (param_count == 3) && (string_param)) {
		if (font_id < 3) {
			for (uns8 count = 0; count < strlen(string_param); count++) {
				string_param[count] = toupper(string_param[count]);
			}	
		}
		if (font_id != 7) { // not bignum
			draw_fonts_print_str(font_id, param_list[1] + x_offset, param_list[2] + y_offset, DRAW_PIXELS_WIDE - param_list[1], 0, param_list[0], string_param);
		} else {
			#ifndef BIGNUM_NOT_SUPPORTED
				draw_bignum(param_list[1] + x_offset, param_list[2] + y_offset, DRAW_PIXELS_WIDE - param_list[1], 0, param_list[0], string_param);
			#endif	
		}
		bad_command = 0;
	} else if (!strcmp(command, "textv") && (param_count == 3) && (string_param)) {
		if (font_id < 3) {
			for (uns8 count = 0; count < strlen(string_param); count++) {
				string_param[count] = toupper(string_param[count]);
			}	
		}
		if (font_id != 7) { // not bignum
			draw_fonts_print_str_vert(font_id, param_list[1] + x_offset, param_list[2] + y_offset, DRAW_PIXELS_WIDE - param_list[1], 0, param_list[0], string_param);
		} else {
			serial_print_str("Vertical print of BigNum not supported\n");	
		}
		bad_command = 0;
	} else if (!strcmp(command, "scroll") && (param_count == 4) && (string_param)) {
		if (font_id < 3) {
			for (uns8 count = 0; count < strlen(string_param); count++) {
				string_param[count] = toupper(string_param[count]);
			}	
		}	
		// here store scroll setup 
		scroll_colour = param_list[0];
		scroll_x_pos = param_list[1] + x_offset;
		scroll_y_pos = param_list[2] + y_offset;
		scroll_width = param_list[3];
		strcpy(&scroll_string, string_param);
		scroll_setup();
		bad_command = 0;
	} else if (!strcmp(command, "paint")) {
		draw_paint();
		bad_command = 0;
	} else if (!strcmp(command, "n")) {
		serial_print_str("Scroll:\n!");
		scroll_display();
		bad_command = 0;
	} else if (!strcmp(command, "test")) {
		draw_tests_run(param_list[0]);
		bad_command = 0;
	} else if (!strcmp(command, "enableactive")) {
		drv_enable_active_when(param_list[0]);
		bad_command = 0;
	} else if (!strcmp(command, "pause")) {
		//pause = 1;
		bad_command = 0;
	} else if (!strcmp(command, "run")) {
		//pause = 0;
		bad_command = 0;
	} else if (!strcmp(command, "r")) {
		drv_refresh();
		bad_command = 0;
	} else if (!strcmp(command, "reset")) {
		asm reset;
	} else if (!strcmp(command, "debug1")) {
		drv_print_buffer();
		bad_command = 0;
	} else if (!strcmp(command, "debug2")) {
		draw_print_buffer();
		bad_command = 0;
	} else if (!strcmp(command, "play")) {
		//recording_playback();
		bad_command = 0;
	}

	if (bad_command) {
		serial_print_str("Bad!\n");
	} else {
		serial_print_str("Ok\n");
		if (recording) {
//			recording_store_command(record_copy);
		}	
	}	

}


	
// configure_system
//
// Get the pic ready for everything we want to do

void configure_system() {
	    
	kill_interrupts();	// turn off interrupts just in case

	turn_analog_inputs_off();	// kill those pesky analogue inputs	
	
	serial_setup(SPBRG_115200);

	term_init();	// initialise terminal 

	// LED setup ----------------------------
	
	make_output(PORTC, 1);	// portc.1 is our led 
	clear_pin(PORTC, 1);	// make it an output, turn it off


	// general setup ----------------------------

	// Timer 0 is used for ticks - 1ms at 64mhz (16mips)
	timer_setup_0(TIMER_16BIT_MODE, TIMER_PRESCALER_OFF, 0xffff - 16000 -1); 

	// Timer 1 is used for led board refresh
	timer_setup_1(TIMER1_PRESCALER_OFF, 0xffff - BOARD_REFRESH -1);

	turn_peripheral_ints_on();
	turn_global_ints_on();

	//draw_fonts_init();

	draw_setup_io();
	draw_init();

}




void main() {
	
uns8 test_result, test_count;
uns16 marker_tick, test_tick;

	configure_system();
	
	delay_ms(100);
	serial_print_str("\n\n");
	serial_print_str(VERSION);
	serial_print_str("\n");
	serial_print_str(BASE_VERSION);
	serial_print_str( "\n<");
	serial_print_str(__TIME__);
	serial_putc(' ');
	serial_print_str(__DATE__);
	serial_print_str(">\n");
	serial_print_str("Supporting up to ");
	serial_print_int(DRAW_DISPLAYS);
	serial_print_str(" displays\n");
	
	/*serial_print_str("Buffer size = ");
	serial_print_int(DRAW_TOTAL_BUFFER_SIZE);
	serial_print_nl();
	serial_print_str("width = ");
	serial_print_int(DRAW_PIXELS_WIDE);
	serial_print_nl();
	serial_print_str("height = ");
	serial_print_int(DRAW_PIXELS_HIGH);
	serial_print_nl();
	serial_print_str("pixels per byte = ");
	serial_print_int(DRAW_PIXELS_PER_BYTE);
	serial_print_nl();
	*/
	set_displays(1);
	timer_start_0();
	timer_start_1();
	
	test_count = 0;
	marker_tick = tick_get_count();	// find out what we're up to
	//draw_fonts_print_str(DRAW_FONT_16TRIPLE_ID, /* x */ 64, /* y */ DRAW_BOTTOM_PIXEL_Y, /* width */ DRAW_PIXELS_WIDE, /* start_pixel */ 0, /* colour */  1, "Hello!");
	draw_fonts_init();
	
	draw_clear_screen();
	draw_paint();
	/*uns8 colour = 1;
	

		for (uns8 count = 0; count < 32; count++) {
			draw_line(0, count, 64*3-1, count, colour);
			draw_paint();
			delay_ms(50);
		}

		for (uns8 count = 0; count < 64; count = count++) {
			draw_line(64*2-1 + count, 0, 64*2-1 + count, 31, colour);
			draw_paint();
			delay_ms(50);
		}
	
	*/

	
	for(;;) {	// for ever
		
		term_process();	// process any typing on the terminal
	
		test_tick = tick_get_count();	// find out what we're up to

		if (tick_calc_diff(marker_tick, test_tick) >= scroll_speed) {
			scroll_display();
			marker_tick = test_tick;
		}

	}
	
}	// main

